@@ -3,7 +3,7 @@ import select, socket, json, threading, random, discord, re
 from os import path, devnull, system, remove
 from sys import stdout, __stdout__
 from time import sleep, time
-from typing import Callable
+from typing import Any, Callable, List, Union
 
 from requests.models import Response
 
@@ -277,11 +277,7 @@ class API:
         if self.valid:
             self.sending = True
             self.__send({"Command":"Is Admin", "Value":uid})
-            while self.buffer == []:
-                sleep(0.1)
-            self.sending = False
-            tmp = self.buffer[0]
-            self.buffer = []
+            tmp = self.__wait_for_response()
             if tmp["Response"] == "Success":
                 return tmp["Data"]
         else: NotValidatedError()
@@ -300,22 +296,14 @@ class API:
         if self.valid:
             self.sending = True
             self.__send({"Command":"Status", "Value":None})
-            while self.buffer == []:
-                sleep(0.1)
-            tmp = self.buffer[0]
-            self.buffer = []
-            self.sending = False
+            tmp = self.__wait_for_response()
             return tmp
         else: raise NotValidatedError()
     
     def get_username(self, key: str) -> str:
         self.sending = True
         self.__send({"Command":'Username', "Value":key})
-        while self.buffer == []:
-            sleep(0.1)
-        self.sending = False
-        tmp = self.buffer[0]
-        self.buffer = []
+        tmp = self.__wait_for_response()
         return tmp["Data"] if tmp["Response"] == "Success" else "unknown"
 
     def send_message(self, message: str, destination: str = None, file_path: str = None) -> bool:
@@ -325,11 +313,7 @@ class API:
         if self.valid:
             self.sending = True
             self.__send({"Command":"Send", 'Value': msg.to_json()})
-            while self.buffer == []:
-                sleep(0.1)
-            tmp = self.buffer[0]
-            self.buffer = []
-            self.sending = False
+            tmp = self.__wait_for_response()
             if tmp["Response"] == "Bad request": raise ActionFailed(tmp["Data"])
             elif tmp["Response"] == "Internal error": 
                 print(f"[Message sending exception] Internal error: {tmp['Data']}")
@@ -365,10 +349,7 @@ class API:
                 self.created_function_list.append([name, help_text, callback])
             if not self.valid: return
             self.__send({"Command":"Create", "Value": [name, help_text, name]})
-            while self.buffer == []:
-                sleep(0.1)
-            tmp = self.buffer[0]
-            self.buffer = []
+            tmp = self.__wait_for_response()
             self.sending = False
             if tmp["Response"] == "Success":
                 self.call_list[name] = callback
@@ -380,22 +361,66 @@ class API:
             self.sending = False
             self.communicateLock.release()
 
+    def __wait_for_response(self) -> Any:
+        while self.buffer == []:
+            sleep(0.1)
+        self.sending = False
+        tmp = self.buffer[0]
+        self.buffer = []
+        return tmp
+
     def connect_to_voice(self, user_id: str) -> bool:
         self.sending = True
         self.__send({"Command": "Connect To User", "Value":user_id})
-        while self.buffer == []:
-            sleep(0.1)
-        self.sending = False
-        tmp = self.buffer[0]
-        self.buffer = 0
-        return True if tmp["Response"] == "Success" else False #TODO FIX ME
+        tmp = self.__wait_for_response()
+        if not tmp["bool"]: print(tmp["Data"])
+        return tmp["bool"]
     
     def disconnect_from_voice(self, user_id: str) -> bool:
         self.sending = True
-        self.__send(self.__send({"Command": "Disconnect From User", "Value":user_id}))
-        while self.buffer == []:
-            sleep(0.1)
-        self.sending = False
-        tmp = self.buffer[0]
-        self.buffer = 0
-        return True if tmp["Response"] == "Success" else False #TODO FIX ME
+        self.__send({"Command": "Disconnect From Voice", "Value":user_id})
+        tmp = self.__wait_for_response()
+        if not tmp["bool"]: print(tmp["Data"])
+        return tmp["bool"]
+
+    def play_file(self, path: str) -> bool:
+        self.sending = True
+        self.__send({"Command": "Play Audio File", "Value": path})
+        tmp = self.__wait_for_response()
+        return tmp == "True"
+
+    def add_file(self, path: str) -> bool:
+        self.sending = True
+        self.__send({"Command": "Add Audio File", "Value": path})
+        tmp = self.__wait_for_response()
+        return tmp == "True"
+
+    def pause_currently_playing(self) -> bool:
+        self.sending = True
+        self.__send({"Command": "Pause Currently Playing", "Value":None})
+        tmp = self.__wait_for_response()
+        return tmp == "True"
+
+    def resume_paused(self) -> bool:
+        self.sending = True
+        self.__send({"Command": "Resume Paused", "Value":None})
+        tmp = self.__wait_for_response()
+        return tmp == "True"
+
+    def skip_currently_playing(self) -> bool:
+        self.sending = True
+        self.__send({"Command": "Skip Currently Playing", "Value":None})
+        tmp = self.__wait_for_response()
+        return tmp == "True"
+
+    def stop_currently_playing(self) -> bool:
+        self.sending = True
+        self.__send({"Command": "Stop Currently Playing", "Value":None})
+        tmp = self.__wait_for_response()
+        return tmp == "True"
+    
+    def get_queue(self) -> Union[List[str], None]:
+        self.sending = True
+        self.__send({"Command": "Skip Currently Playing", "Value":None})
+        tmp = self.__wait_for_response()
+        return tmp
